<?php

namespace Drupal\hook_watch\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Finder\Finder;
use Symfony\Component\HttpKernel\KernelEvents;
use Yosymfony\ResourceWatcher\Crc32ContentHash;
use Yosymfony\ResourceWatcher\ResourceCachePhpFile;
use Yosymfony\ResourceWatcher\ResourceWatcher;

/**
 * Main event subscriber.
 *
 * @package Drupal\hook_watch\EventSubscriber
 */
class KernelEventSubscriber implements EventSubscriberInterface {

  /**
   * Executed at the beginning of every request.
   *
   * @todo Only clear the specific cache items that we need.
   */
  public function onRequest() {
    $finder = new Finder();
    $finder->files()
      ->name('*.module')
      ->in(DRUPAL_ROOT . '/modules/');

    $hashContent = new Crc32ContentHash();
    $resourceCache = new ResourceCachePhpFile(\Drupal::service('file_system')
      ->realpath('private://') . '/hook_watch_cache.php');
    $watcher = new ResourceWatcher($resourceCache, $finder, $hashContent);
    $watcher->initialize();

    $result = $watcher->findChanges();
    if ($result->hasChanges()) {
      \Drupal::cache('default')->deleteAll();
      \Drupal::cache('bootstrap')->deleteAll();
    }
  }

  /**
   * Subscribe to events.
   *
   * @return array
   *   Subscribed events
   */
  public static function getSubscribedEvents() {
    return [
      // Set high priority value to start as early as possible.
      KernelEvents::REQUEST => ['onRequest', 1001],
    ];
  }

}
